----------------------------------------------------------------------------------
-- Engineer1: Programmcode Ralph Hnsel Stand 04.11.2011 TFT_KOMBI
-- Engineer2: Farbanbau Heiko_Poppe     Stand 31.08.2011
-- Module Name: BWS_TFT_Kombi
-- Taktgenerator 40Mhz
-- Features:
-- TFT <=> FBAS Modus via Pin TFT_Mode H = TFT Mode
-- BWS Steuerregister IOF0H ist rcklesbar !
-- Taktumschaltung am Takt-Pin  IO F0H Bit 0 0 = 2Mhz     1 = 4Mhz
-- INVERS Modus BWS             IO F0H Bit 1 0 = Normal   1 = INVERS
-- FarbRAM (1000..17FFH) aktiv  IO F0H Bit 2 0 = Vid.RAM  1 = FarbRAM
-- ZG ProgrammierModus          IO F0H Bit 3 0 = Normal   1 = Programmiermode
-- ZG Zeichenadressen / Reserve IO F0H Bit 4..7
-- ZG  Umschaltung ACC<=> SCCH  wird erkannt (via PIO1 Port B3)
-- INVERS Erkennung von Zeichen 11H/10H
-- 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL ; --notwendig fr die Zhler !

entity BWS_TFT_Kombi is
   Port ( 
			DB          : inout STD_LOGIC_VECTOR (7 downto 0);  -- Datenbus Bidirektional
			ADR         : in  STD_LOGIC_VECTOR (15 downto 11);  -- Adresse A15..A11
			ADR7        : in  STD_LOGIC_VECTOR (7 downto 0);    -- Adresse A7..A0
			MREQ        : in  STD_LOGIC;								
			WR          : in  STD_LOGIC;
			RD          : in  STD_LOGIC;
			Reset       : in  STD_LOGIC;
			IORQ        : in  STD_LOGIC;
			MEMDI_Mod1  : in  STD_LOGIC;                        -- L wenn Speicher deaktiviert 
			MEMDI_AC1   : in  STD_LOGIC;								 -- L wenn AC1 + Modul1 deaktiviert. dieses
																				 -- Signal ist das "CPM ein" vom AC1 2010
			TFT_Mode    : in  STD_LOGIC;                        -- H=TFT Mode, L=Normal AC1 FBAS Mode
			RAMADR	   : out    STD_LOGIC_VECTOR (10 downto 0);-- A10..A0 BWS AusleseRAM (links)
			RAMDB		   : inout  STD_LOGIC_VECTOR (7 downto 0) ;-- A10..A3 ZG = BWS DB AusleseRAM (fr Anzeige)
         ZGADR		   : out STD_LOGIC_VECTOR (2 downto 0);    -- A2..A0 ZG (Zeilenadresse Zeichen)
			ZGADR11     : out STD_LOGIC;                        -- Umschaltung ZG ACC<-->SCCH Modus
			CSREADRAM   : out STD_LOGIC;								 -- CS Signal fr RAM & FarbRAM (links) AusleseRAM
			CSRAM		   : out STD_LOGIC;								 -- CS Signal fr RAM     (rechts) CPU-Seite
			CSFARBRAM   : out STD_LOGIC;								 -- CS Signal fr FarbRAM (rechts) CPU-Seite
			CSFARBLATCH : out STD_LOGIC;                        -- Datenuebernahme Latch aus FarbRAM
			CSWR        : out STD_LOGIC;  							 -- WR Signal fr ZG
         ZG_Out      : inout  STD_LOGIC_VECTOR (5 downto 0); -- Ausgnge ZG (intern an Schieberegister)
			DTVDU		   : inout STD_LOGIC;                      -- Dunkeltastung Bildraender
         Video_out   : out STD_LOGIC;                        -- VideoSignal        negiert (wegen Treiber)			
			HK_Sync 	   : out STD_LOGIC;                        -- HK SynchronImpuls  negiert (wegen Treiber)
			VK_Sync 	   : out STD_LOGIC;                        -- VK SynchronImpuls  negiert (wegen Treiber)
			Takt	 	   : buffer STD_LOGIC;                     -- AC1 2010 Ausgangstakt, 2Mhz/4Mhz 
			Takt_in 	   : in  STD_LOGIC                         -- 40 Mhz Eingangstakt
			);

end BWS_TFT_Kombi;

architecture VERHALTEN of BWS_TFT_Kombi is
	signal   VDCLK       : STD_LOGIC :=	'0';                         -- VDCLK = Bildpunkttakt
	signal	Teiler2_40	: UNSIGNED (0 downto 0) := (others => '0'); -- Teiler 2:1 fr TFT Mode
	signal	Teiler5_40	: UNSIGNED (2 downto 0) := (others => '0'); -- Teiler 5:1 fr FBAS Mode
	signal	Teiler4_8	: UNSIGNED (1 downto 0) := (others => '0'); -- 4:1 Teiler fr CPU Takt
	signal	QD19			: UNSIGNED (2 downto 0) := (others => '0'); -- Ausgang D19 6:1 Taktteiler
	signal	QD41_42		: UNSIGNED (6 downto 0)	:= (others => '0'); -- Ausgang D41 und 42 Zeichenzhler
	signal	QTFT			: UNSIGNED (0 downto 0) := (others => '0'); -- Ausgang QTFT 2:1 Teiler fr TFT Modus
	signal	QD43_44_20	: UNSIGNED (8 downto 0)	:= (others => '0'); -- Ausgang D43,D44,D20 Zhler
	signal	BWS_Zugriff : STD_LOGIC := '0';                         -- Signal 1 fr Zugriff der CPU auf BWS
	signal	BWS_aus		: STD_LOGIC := '1';                         -- Signal 0 wenn BWS ausgeschaltet
	signal	D35_36		: STD_LOGIC_VECTOR(7 downto 0) := (others => '0') ;-- internes Latch zum Zeichen speichern
	signal   SR_ZG_Out   : STD_LOGIC_VECTOR (5 downto 0) := (others => '0');-- Ausgnge Schieberegister ZG
	signal	DG13			: STD_LOGIC :=	'0'  ;                              -- Dunkeltastung = 1 bei Strahlrcklauf
	signal 	BWS_INV		: STD_LOGIC := '0'  ;                              -- BWS INVERS Umschaltung
	signal	Zei_INV		: STD_LOGIC := '0'  ;                              -- Zeichen INVERS Umschaltung
   signal   StartZeile  : STD_LOGIC := '0'  ; -- Signal 1 Start neue Zeile
   signal   IORQ_WR_05H : STD_LOGIC := '0'  ; -- Signal 1 wenn IO Port 05H geschrieben wird
   signal 	IO5_Latch   : STD_LOGIC := '0'  ; -- PIO Port B ZG Umschaltung
   signal   IORQ_WR_F0H : STD_LOGIC := '0'  ; -- Signal 1 wenn IO Port F0H geschrieben wird
   signal 	IOF0_Latch  : STD_LOGIC_VECTOR(7 downto 0) := (others => '0') ;-- LatchIOF0 
-- ***********************************************************************************************************

begin
-- Zuerst die ntigen Systemtakte erzeugen, an "Takt_In" liegen 40Mhz an, 
-- fr TFT Modus ist VDCLK=20Mhz und Normalmodus (FBAS)=8Mhz ntig
process(Takt_In)
  begin
    if rising_edge(Takt_In) then Teiler2_40 <= Teiler2_40 + 1 ; -- Zhlen 0,1
    end if;
-- hier kommen jetzt bei STD_LOGIC(Teiler2_40) 20Mhz raus
end process; 


-- hier nun Taktteiler 5:1, um VDCLK 8Mhz fr BAS Modus zu erzeugen
process(Takt_In)
begin
	if rising_edge(Takt_In) then
				if Teiler5_40  < 4 then				  -- 0..4 zhlen = 5:1 teilen
					Teiler5_40  <= Teiler5_40 + 1 ; -- zhlen
				else
					Teiler5_40   <= "000" ;			  -- rcksetzen
				end if ; 
	end if;
-- hier kommen jetzt bei STD_LOGIC(Teiler5_40(2)) 8Mhz raus	
end process;

-- Taktteiler 4:1, um aus 8Mhz 2/4Mhz fr CPU Takt zu erzeugen
process(Takt_In)
begin
	if rising_edge(Takt_In) then
		if	Teiler5_40 = 4 then						-- Freigabe Zhler	
				if Teiler4_8  < 3 then				-- 0..3 zhlen = 4:1 teilen
					Teiler4_8  <= Teiler4_8 + 1  ;-- zhlen
				else
					Teiler4_8   <= "00" ;			-- rcksetzen
				end if ; 
		end if;
	end if;
end process ;

-- hier jetzt das ZENTRALE Taktsignal fr alle Prozesse definieren,

	VDCLK		<= STD_LOGIC(Teiler2_40(0))   -- bei 20Mhz Takt hier Takt_in 
			when  TFT_Mode = '1'							-- TFT Modus (20Mhz)
	        else STD_LOGIC(Teiler5_40(2));			-- FBAS Modus (8Mhz)


-- CPU Takt je nach IOF0_Latch Bit 0=2Mhz 1=4Mhz erzeugen
	Takt		<=	STD_LOGIC(Teiler4_8(0)) when IOF0_Latch(0) = '1'     -- 4Mhz
													else STD_LOGIC(Teiler4_8(1));-- 2Mhz	

-- IO5 Latch fngt Ausgaben an PIO Port B ab (ZG Umschaltung)
IORQ_WR_05H <= '0' when ( IORQ = '0' and WR = '0' and ADR7(7 downto 0) = x"05" ) else '1';

process (Reset, IORQ_WR_05H) 
begin
		 if Reset = '0'              then IO5_Latch  <= '0'  ; -- Reset auswerten
    elsif rising_edge(IORQ_WR_05H) then IO5_Latch  <= DB(3); -- Latch vom DB laden
    end if;
end Process;

--process(VDCLK)
--begin
--	if	rising_edge(VDCLK) then 
--		if	(ADR7(7 downto 0) = x"05" and IORQ = '0' and WR = '0')-- IO Adresse PIO1 Port B
--			then IO5_Latch  <= DB(3)       ;								-- Bit 3 (PIO 1 PortB) ZG Modeumschaltung
--			else IO5_Latch  <= IO5_Latch	 ;								-- Zustand behalten
--		end if;	
--	end if;
--end process;

-- hier gleich die Zeichensatzumschaltung ZG ACC <==>SCCH 1=SCCH
   ZGADR11		<=	IO5_Latch ; 	-- ZG-Umschaltung 

-- IOF0H Register mit 8 Bit Breite (DB7..0) Bit0 Takt, Bit1 INVERS, Bit2 Umschaltung Farbe
--                                          Bit3 ZG Programmiermode, Bit4..6 ZG Adr A0..A2  
IORQ_WR_F0H <= '0' when ( IORQ = '0' and WR = '0' and ADR7(7 downto 0) = x"F0" ) else '1';

process (Reset, IORQ_WR_F0H) 
begin
		 if Reset = '0'              then IOF0_Latch  <= (others =>'0') ; -- Reset auswerten
    elsif rising_edge(IORQ_WR_F0H) then IOF0_Latch  <= DB(7 downto 0) ; -- Latch vom DB laden
    end if;
end Process;


	
-- Daten vom Farb-RAM ins Farb_Latch einlesen
process (VDCLK)
begin
	if falling_edge(VDCLK) then
		if QD19 = 0							-- Timing Zeichen / Hintergrund auf 0
			then	CSFARBLATCH	<=	'1' ;
			else  CSFARBLATCH <= '0' ; -- Taktsignal fr Latch 74LS374 L/H speichert
		end if;
	end if;
end process;


-- Signal BWS_aus bilden, immer wenn der BWS mittels MEMDI deaktiviert wird
	BWS_aus	<=	'1'	when	MEMDI_AC1  = '0' 		-- CPM Zugriff
								or 
									MEMDI_Mod1 = '0'		-- Modul 3 Zugriff
								else '0' ; 

-- Daten vom Zeichen-RAM in internes Latch D35_36 (wg.INVERS) einlesen
process(VDCLK)
begin
     if rising_edge(VDCLK) then
         if QD41_42 < 64 then    -- nur Zeichen im sichtbaren Bereich einlesen
              if QD19 = 0 then D35_36 <= RAMDB  ; -- Zeichen einlesen
                          else D35_36 <= D35_36 ; -- Latch bilden
             end if;
         end if;
     end if;
end process;

-- ZG Daten ins Schieberegister SR_ZG_Out holen und herausschieben
ZG_Out_ausschieben: 	
process(VDCLK)  				-- Zeichentakt
	begin
	if rising_edge(VDCLK) then
      if QD19 = 5 then  SR_ZG_Out(5 downto 0) <= ZG_Out(5 downto 0) ; -- parallel laden      
					 	 else SR_ZG_Out <= '0' & SR_ZG_Out(5 downto 1) ;  	 -- serial schieben
		end if;
   end if;
end process ZG_Out_ausschieben;


-- Adressdekodierung & Zugriff selektieren 1000H..17FFH
 	BWS_Zugriff <= '1' when ADR(15 downto 12) =x"1" -- Adresse 1xxxH 
								and ADR(11)='0' 				-- Adresse 1000H..17FFH
								and MREQ='0' 							-- Speicherzugriff
								and BWS_aus = '0'						-- wenn BWS aktiv sein darf
								else '0' ; 						      -- BWS auslesen

-- CS fr BWS RAM erzeugen
	CSRAM       <= '0' when  BWS_Zugriff = '1' 		-- normaler BWSzugriff
								and IOF0_Latch(2) = '0' 	-- FarbRam inaktiv
								and IOF0_Latch(3) = '0' 	-- ZG Programmiermodus inaktiv
							 else '1' ;							-- RAM BWS inaktiv
-- CS fr Farb RAM erzeugen
	CSFARBRAM   <= '0' when BWS_Zugriff = '1' 		-- normaler BWSzugriff
								and IOF0_Latch(2) = '1'		--	FarbRAM aktiv
								and IOF0_Latch(3) = '0' 	-- ZG Programmiermodus inaktiv
							  else '1' ;						-- RAM Farbe inaktiv

-- Programmierung Auslesen ZG wenn IOF0(3)=1
	CSREADRAM 	<= '1' when IOF0_Latch(3) = '1' 		-- BWS RAM's inaktive wenn ZG Prog.Modus
								else '0' ;				  		-- BWS RAM's aktiv
-- WR Signal fr ZG (EEPROM 28C764)
	CSWR			<= '0' when (IOF0_Latch(3) = '1'		-- ZG Programmiermode
								and BWS_Zugriff   = '1' 	-- BWS Zugriff (WR nur im BWS RAM)
								and WR            = '0' )  -- ZG-Modus + BWS Zugriff
								else '1' ;				  		-- ZG inaktiv

-- ZG Schreibdaten vom Datenbus an ZG legen
	ZG_OUT	<= DB(5 downto 0) when (IOF0_Latch(3) = '1'  -- ZG Programmiermode
											and BWS_Zugriff  = '1'  -- BWSZugriff
											and WR           = '0') -- WR Daten ZG
										ELSE "ZZZZZZ" ;				-- ZG_OUT hochohmig		

-- Daten auf DB je nach Zustand
	DB			<= IOF0_LATCH	  when (ADR7(7 downto 0) = x"F0"  -- Adresse F0H
											and           IORQ = '0' 	 -- IORQ aktiv
											and 				 RD = '0')	 -- IOF0H rcklesen
										ELSE 
					"00" & ZG_OUT when (IOF0_Latch(3)    = '1' 	 -- ZG Programmiermode
											and BWS_Zugriff    = '1' 	 -- BWS Zugriff
											and RD             = '0')   -- ZG-Mode (auslesen)
										ELSE "ZZZZZZZZ" ;					 -- DB hochohmig

-- RAM Adressen bilden ------------------------------------------------
	RAMADR(5 downto 0)  <= not STD_LOGIC_VECTOR(QD41_42(5 downto 0))    ; -- inv. wg. negiertem Adressbus
	RAMADR(10 downto 6) <= not STD_LOGIC_VECTOR(QD43_44_20(7 downto 3)) ;

-- ZG Adresse erzeugen, wenn ZG Mode (IOF0 Bit 3=1)
	ZGADR(2 downto 0)  <= IOF0_LATCH(6 downto 4) when IOF0_Latch(3) = '1' -- ADR ZG Lesen/Schreiben
								else STD_LOGIC_VECTOR(QD43_44_20(2 downto 0)) ;  -- Bits 0..2 D43 Zeichen

	RAMDB					 <= ADR7(7 downto 0) when IOF0_Latch(3) = '1' -- A3..A10 ZG vom Adressebus A0..A7
								else "ZZZZZZZZ" ;									 -- hochohmig

-- INVERS Modus 10H/11H 
process(Takt_In,VDCLK)
begin
     if rising_edge(VDCLK) then
         if QD19 = 3 then                -- Zeitpunkt der Umschaltung (ca. Zeichenmitte)
             if D35_36 = x"11" then Zei_INV <= '1' ;
             else
                 if D35_36 = x"10"
                 or QD41_42 = 66 then Zei_INV <= '0' ;
                 else Zei_INV <= Zei_INV ;
                 end if ;
             end if ;
         end if ;
     end if ;
end process ;

-- INVERS-Auswertung	
	BWS_Inv     <= IOF0_Latch(1) when Zei_INV ='0'	   	 -- Umschaltung INVERS
											else not IOF0_Latch(1) ; -- Umschaltung Invertieren	

-- Hinweis.. durch die Treiber nach dem CPLD muss die Invertierung des Signales beachtet werden! 
	Video_Out <= '1' when DTVDU = '1' 									-- Dunkeltastung 1=dunkel
								or IOF0_Latch(3) = '1'                 -- ZG ProgrammierModus
							else SR_ZG_out(0) when BWS_INV = '1' 		-- INVERS oder NICHT INVERS
													 else not SR_ZG_out(0) ;-- Ausgang Schieberegister
													 
-- Signal DG13 bilden, es stellt die Zeit des Strahlrcklaufes dar
process(VDCLK)
begin
	if	falling_edge(VDCLK) then
				if    QD41_42 = 64 and QD19 = 0 then DG13 <= '1' ; -- Dunkeltastung
				elsif QD41_42 = 0  and QD19 = 0 then DG13 <= '0' ; -- Anzeige aktiv
				end if;
	end if;
end process;

-- Signal DTVDU bilden.. (Dunkeltastsignal fr die Rnder & Austastlcken) und
--								  1 wenn ZG programmiert wird
	DTVDU	<= '0' when  (DG13 = '0' 			  				-- Ausgang FlipFlop DG13 (Strahlrcklauf)
						and	QD43_44_20(8)= '0'   			-- invertierter D20 (Seitenrnder)
						and   IOF0_Latch(3)= '0')           -- kein ZG Programmiermode
						or (IOF0_Latch(3)= '1' and RD= '0') -- ZG Programmiermode --> ZG lesen
					  else '1' ;			

-- *** ab hier Zhlerkette ***************************************************
-- Hinweis: Die Zhlerkette erzeugt ein SVGA Signal bei 60Hz

-- D19 (Basis ist VDCLK) mit 20Mhz bei TFT Mode, oder 8Mhz bei BAS Mode
process(VDCLK)     -- Takt wird durch 6=6 Zeilen je Zeichen geteilt 
  begin
    if rising_edge(VDCLK) then 
		if QD19 < 5 then  		-- zhlt 0..5
		  QD19    <= QD19 + 1 ;	-- zhlen
		else
		  QD19    <= "000"; 		-- rcksetzen
			end if ; 
    end if;
  end process;

-- Spaltenzhler D41_D42 zhlt Zeichen 1..87, Leerrume & HK-Synchronimpuls
D41_42: 
process(VDCLK)
	begin
		if rising_edge(VDCLK) then
			if QD19 = 3 then					   -- Zhlfreigabe 	
				if QD41_42   < 87 then			-- bis 87 zhlen
					QD41_42   <= QD41_42 + 1 ; -- zhlen
				else
					QD41_42   <= "0000000"   ; 
				end if ; 
			end if ;
		end if ;
end process D41_42;				

TFT_Zeile2: 	-- 2:1 Teiler sorgt dafr, dass jede Zeile 2x geschrieben wird
process(VDCLK)
	begin
		if rising_edge(VDCLK) then
		if QD41_42 = 83 and QD19 = 3	then	-- Freigabe Zhler	
				if QTFT   = 0 then				-- bis 1 zhlen
					QTFT   <= QTFT + 1 ; 		-- zhlen
				else
					QTFT   <= "0"   ; 
				end if ; 
			end if ;
		end if ;
end process TFT_Zeile2;				

-- Hilfssignal Startzeile, bestimmt wann Zeilenzhler weiterschaltet
	StartZeile <= '1' when   
								(TFT_Mode = '0'		-- FBAS Modus BWS
								and QD19    = 3
								and QD41_42 = 82)
							or
								(TFT_Mode = '1'		-- TFT Modus BWS
								and QD19    = 3
								and QD41_42 = 82
								and QTFT    = 1)
							else '0' ;	

-- ZeilenZhler D43 & D44 & D20 zhlt Zeilen 0 bis 313
D43_44_20:
process(VDCLK)
begin
	if rising_edge(VDCLK)  then 
		if StartZeile = '1' then	   	 							-- Freigabe Zhler
			if QD43_44_20 < 313 then 									-- bis 313
										QD43_44_20 <= QD43_44_20 + 1 ;-- zhlen..
										else
										QD43_44_20 <= "000000000" 	  ;-- rcksetzen
			end if ;
		end if;
	end if;	
end process D43_44_20;
-- *** BWS Zhlerkette --Ende----------------------------------------------

-- ******** Erzeugung HK Synchronimpuls aus Zhlerkette *******************

-- HK Impuls startet bei Zhlerstand 69 von QD41_42 und 11 Takte lang 
-- HK Impuls sollte 4,7us=4700ns lang sein
process(VDCLK)
begin
	if rising_edge(VDCLK) then
									if     QD41_42 = 69 then -- 69 Start HK Impuls
											HK_sync <= '1';
									elsif  QD41_42 = 80 then -- 80 Ende HK Impuls
											HK_sync <= '0';	
									end if;
	end if;
end process;
-- ***Ende HK_Sync Impuls ****************************************************

-- ******** Erzeugung VK Synchronimpuls aus Zhlerkette **********************
-- VK Impuls startet bei Zhlerstand 278 von QD43_44_20 und ist 2 Taket lang
-- VK Impls sollte 2,5x64uS=160us(=160.000ns) breit sein
process(VDCLK)
begin
	if rising_edge(VDCLK) then
									if 	QD43_44_20 	= 278 then -- 278 Start VK Impuls
												VK_sync <= '1';
									elsif QD43_44_20 	= 280 then -- 280 Ende VK Imuls
												VK_sync <= '0';
									end if;
	end if;
end process;
-- ***Ende VK_Sync Impuls ****************************************************
end VERHALTEN;